class IAMPolicies:
    CAN_VIEW_OTHERS_OPERATIONS = 'CAN_VIEW_OTHERS_OPERATIONS'
    CAN_VIEW_OTHERS_WORKFLOWS = 'CAN_VIEW_OTHERS_WORKFLOWS'
    CAN_VIEW_OPERATIONS = 'CAN_VIEW_OPERATIONS'
    CAN_VIEW_WORKFLOWS = 'CAN_VIEW_WORKFLOWS'

    CAN_CREATE_OPERATIONS = 'CAN_CREATE_OPERATIONS'
    CAN_CREATE_WORKFLOWS = 'CAN_CREATE_WORKFLOWS'

    CAN_MODIFY_OTHERS_WORKFLOWS = 'CAN_MODIFY_OTHERS_WORKFLOWS'

    CAN_RUN_WORKFLOWS = 'CAN_RUN_WORKFLOWS'

    IS_ADMIN = 'IS_ADMIN'


class OperationViewSetting:
    KIND_AND_TITLE = 'KIND_AND_TITLE'
    TITLE_AND_DESCRIPTION = 'TITLE_AND_DESCRIPTION'


class UserPostAction:
    MODIFY = 'MODIFY'


class RegisterUserExceptionCode:
    EMPTY_USERNAME = 'EMPTY_USERNAME'
    EMPTY_PASSWORD = 'EMPTY_PASSWORD'
    USERNAME_ALREADY_EXISTS = 'USERNAME_ALREADY_EXISTS'
    EMAIL_ALREADY_EXISTS = 'EMAIL_ALREADY_EXISTS'
    INVALID_EMAIL = 'INVALID_EMAIL'
    INVALID_LENGTH_OF_USERNAME = 'INVALID_LENGTH_OF_USERNAME'



import plynx.constants
import plynx.db.node
import plynx.utils.plugin_manager

workflow_manager = plynx.utils.plugin_manager.get_workflow_manager()
executor_manager = plynx.utils.plugin_manager.get_executor_manager()


def create_template(user, kind, cmd, title, description, inputs=None, parameters=None, outputs=None):
    node = executor_manager.kind_to_executor_class[kind].get_default_node(
        is_workflow=kind in workflow_manager.kind_to_workflow_dict
    )
    node.author = user._id
    node.title = title
    node.description = description
    node.kind = kind

    cmd_parameter = next(filter(lambda parameter: parameter.name == '_cmd', node.parameters))
    cmd_parameter.value.value = cmd

    node.inputs.extend(inputs or [])
    node.parameters.extend(parameters or [])
    node.outputs.extend(outputs or [])

    import logging
    from plynx.utils.common import JSONEncoder
    logging.info(JSONEncoder().encode(node.to_dict()))

    validation_error = executor_manager.kind_to_executor_class[kind](node).validate()
    if validation_error:
        raise Exception('Validation failed')

    node.node_status = plynx.constants.NodeStatus.READY

    node.save(force=True)


def create_default_templates(user):
    create_template(
        user=user,
        kind='basic-bash-jinja2-operation',
        cmd='cat {{inputs["in"] | join(" ")}} | paste -sd+ | bc >> {{outputs["out"]}}',
        title='Sum of numbers',
        description='sum',
        inputs=[
            plynx.db.node.Input({
                'name': 'in',
                'is_array': True,
            }),
        ],
        outputs=[
            plynx.db.node.Output({
                'name': 'out',
            }),
        ],
    )

    create_template(
        user=user,
        kind='basic-bash-jinja2-operation',
        cmd='seq {{params["from"]}} {{params["to"]}} >> {{outputs["out"]}}',
        title='Numbers A to B',
        description='Print text',
        parameters=[
            plynx.db.node.Parameter({
                'name': 'from',
                'type': plynx.constants.ParameterTypes.INT,
                'value': '1',
                'widget': 'From',
            }),
            plynx.db.node.Parameter({
                'name': 'to',
                'type': plynx.constants.ParameterTypes.INT,
                'value': '100',
                'widget': 'To',
            }),
        ],
        outputs=[
            plynx.db.node.Output({
                'name': 'out',
            }),
        ],
    )



import json
from flask import request, send_file, g
import plynx.db.node
from plynx.web.common import app, requires_auth, make_success_response, make_fail_response, handle_errors
import plynx.base.resource
from plynx.plugins.resources.common import FILE_KIND
import plynx.utils.plugin_manager
from plynx.utils.file_handler import get_file_stream, upload_file_stream
from plynx.constants import NodeRunningStatus, NodeStatus


RESOURCE_TYPES = list(plynx.utils.plugin_manager.get_resource_manager().kind_to_resource_class.keys())


@app.route('/plynx/api/v0/resource/<resource_id>', methods=['GET'])
@handle_errors
def get_resource(resource_id):
    preview = json.loads(request.args.get('preview', 'false'))
    file_type = request.args.get('file_type', None)
    if preview and not file_type:
        return make_fail_response('In preview mode `file_type` must be specified'), 400
    fp = get_file_stream(resource_id, preview=preview, file_type=file_type)
    if preview:
        preview_object = plynx.base.resource.PreviewObject(
            fp=fp,
            resource_id=resource_id,
        )
        return plynx.utils.plugin_manager.get_resource_manager().kind_to_resource_class[file_type].preview(preview_object)
    return send_file(
        fp,
        attachment_filename=resource_id)


@app.route('/plynx/api/v0/resource', methods=['POST'])
@handle_errors
@requires_auth
def post_resource():
    resource_id = upload_file_stream(request.files['data'])
    return make_success_response({
        'resource_id': resource_id
    })


@app.route('/plynx/api/v0/upload_file', methods=['POST', 'PUT'])
@handle_errors
@requires_auth
def upload_file():
    assert len(request.files) == 1
    title = request.form.get('title', '{title}')
    description = request.form.get('description', '{description}')
    file_type = request.form.get('file_type', FILE_KIND)
    node_kind = request.form.get('node_kind', 'basic-file')
    app.logger.debug(request)
    if file_type not in RESOURCE_TYPES:
        app.logger.debug(file_type)
        app.logger.debug(RESOURCE_TYPES)
        return make_fail_response('Unknown file type `{}`'.format(file_type)), 400

    resource_id = upload_file_stream(request.files['data'])

    file = plynx.db.node.Node.from_dict({
        'title': title,
        'description': description,
        'kind': node_kind,
        'node_running_status': NodeRunningStatus.STATIC,
        'node_status': NodeStatus.READY,
    })
    file.outputs.append(
        plynx.db.node.Output.from_dict({
            'name': 'file',
            'file_type': file_type,
            'values': [resource_id],
        })
    )

    file.author = g.user._id
    file.save()

    return make_success_response({
        'resource_id': resource_id,
        'node': file.to_dict(),
    })



class NodeNotFound(ImportError):
    pass


class RegisterUserException(Exception):
    def __init__(self, message, error_code):
        super().__init__(message)
        self.message = message
        self.error_code = error_code
